/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: JORAMServer.java 2332 2009-05-26 10:34:29Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.server.impl.jms;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Arrays;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.objectweb.joram.client.jms.admin.AdminModule;

import fr.dyade.aaa.agent.AgentServer;

/**
 * Handle an embedded JORAM server.
 * @author Loris Bouzonnet
 */
public final class JORAMServer {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(JORAMServer.class);

    /**
     * The configuration of JMS implementation.
     */
    private final Config config;


    /**
     * Create a new handler of JORAM server.
     * @param config the configuration of JMS implementation.
     */
    public JORAMServer(final Config config) {
        this.config = config;
    }


    /**
     * Start the JORAM server.
     * @throws JORAMServerException if the JORAM server cannot be started or configured.
     */
    public void start() throws JORAMServerException {

        if(config.getServerId() == null) {
            logger.error("Unable to start a JORAM server without server id");
            throw new JORAMServerException("Unable to start a JORAM server without server id");
        }
        String[] args = {Short.toString(config.getServerId()), "./s" + config.getServerId()};
        try {
            fr.dyade.aaa.agent.AgentServer.main(args);
        } catch (Exception e) {
            logger.error("Unable to start the JORAM server with the args "
                    + Arrays.toString(args), e);
            throw new JORAMServerException("Unable to start the JORAM server with the args "
                    + Arrays.toString(args), e);
        }

        InputStream is =
            Thread.currentThread().getContextClassLoader().getResourceAsStream("joramAdmin.xml");
        if (is != null) {
            Reader reader = null;
            try {
                reader = new InputStreamReader(is);
                AdminModule.executeAdmin(reader);
            } catch (Exception e) {
                logger.error("Unable to administrate the JORAM server", e);
                throw new JORAMServerException("Unable to administrate the JORAM server", e);
            } finally {
                try {
                    if (reader != null) {
                        reader.close();
                    }
                } catch (IOException e) {
                    logger.warn("Unable to close the reader of joramAdmin.xml", e);
                }
            }
        }

        try {
            AdminModule.collocatedConnect("root", "root");
        } catch (Exception e) {
            logger.error("Unable to connect to the JORAM server", e);
            throw new JORAMServerException("Unable to connect to the JORAM server", e);
        }

    }

    /**
     * Stop the JORAM server.
     */
    public void stop() {
        AgentServer.stop();
    }

}
